@@ -333,7 +333,7 @@ add_themes_from_dir (DialogData *data,
 			continue;
 		}
 
-		if (g_strcmp0 (_g_uri_get_file_extension (g_file_info_get_name (file_info)), ".cst") != 0) {
+		if (g_strcmp0 (_g_path_get_extension (g_file_info_get_name (file_info)), ".cst") != 0) {
 			g_object_unref (file_info);
 			continue;
 		}
@@ -724,7 +724,7 @@ dlg_contact_sheet (GthBrowser *browser,
 	gth_browser_set_dialog (browser, "contact_sheet", data->dialog);
 	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);
 
-	data->thumbnail_caption_chooser = gth_metadata_chooser_new (GTH_METADATA_ALLOW_IN_PRINT);
+	data->thumbnail_caption_chooser = gth_metadata_chooser_new (GTH_METADATA_ALLOW_IN_PRINT, TRUE);
 	gtk_widget_show (data->thumbnail_caption_chooser);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("thumbnail_caption_scrolledwindow")), data->thumbnail_caption_chooser);
 
@@ -743,7 +743,7 @@ dlg_contact_sheet (GthBrowser *browser,
 	else
 		s_value = _g_settings_get_uri (data->settings, PREF_CONTACT_SHEET_DESTINATION);
 	if (s_value == NULL)
-		s_value = g_strdup (get_home_uri ());
+		s_value = g_strdup (_g_uri_get_home ());
 	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
 	g_free (s_value);
 
