@@ -25,5 +25,7 @@
 #include <gthumb.h>
 
 DEF_ACTION_CALLBACK (gth_browser_activate_video_screenshot)
+DEF_ACTION_CALLBACK (gth_browser_activate_toggle_play)
+DEF_ACTION_CALLBACK (gth_browser_activate_video_zoom_fit)
 
 #endif /* ACTIONS_H */
