@@ -23,6 +23,7 @@
 #include <gthumb.h>
 #include <gdk/gdkkeysyms.h>
 #include "gth-script.h"
+#include "shortcuts.h"
 
 
 static void gth_script_dom_domizable_interface_init (DomDomizableInterface *iface);
@@ -42,12 +43,6 @@ enum {
 };
 
 
-typedef struct {
-	guint            keyval;
-	GdkModifierType  modifiers;
-	char            *name;
-} _Accel;
-
 struct _GthScriptPrivate {
 	char            *id;
 	char            *display_name;
@@ -56,7 +51,8 @@ struct _GthScriptPrivate {
 	gboolean         shell_script;
 	gboolean         for_each_file;
 	gboolean         wait_command;
-	_Accel           accelerator;
+	char            *accelerator;
+	char            *detailed_action;
 };
 
 
@@ -88,7 +84,6 @@ gth_script_real_create_element (DomDomizable *base,
 					       "shell-script", (self->priv->shell_script ? "true" : "false"),
 					       "for-each-file", (self->priv->for_each_file ? "true" : "false"),
 					       "wait-command", (self->priv->wait_command ? "true" : "false"),
-					       "accelerator", self->priv->accelerator.name,
 					       NULL);
 	if (! self->priv->visible)
 		dom_element_set_attribute (element, "display", "none");
@@ -114,7 +109,7 @@ gth_script_real_load_from_element (DomDomizable *base,
 		      "shell-script", (g_strcmp0 (dom_element_get_attribute (element, "shell-script"), "true") == 0),
 		      "for-each-file", (g_strcmp0 (dom_element_get_attribute (element, "for-each-file"), "true") == 0),
 		      "wait-command", (g_strcmp0 (dom_element_get_attribute (element, "wait-command"), "true") == 0),
-		      "accelerator", dom_element_get_attribute (element, "accelerator"),
+		      "accelerator", "",
 		      NULL);
 }
 
@@ -134,7 +129,7 @@ gth_script_real_duplicate (GthDuplicable *duplicable)
 		      "shell-script", script->priv->shell_script,
 		      "for-each-file", script->priv->for_each_file,
 		      "wait-command", script->priv->wait_command,
-		      "accelerator", script->priv->accelerator.name,
+		      "accelerator", script->priv->accelerator,
 		      NULL);
 
 	return (GObject *) new_script;
@@ -150,12 +145,28 @@ gth_script_finalize (GObject *base)
 	g_free (self->priv->id);
 	g_free (self->priv->display_name);
 	g_free (self->priv->command);
-	g_free (self->priv->accelerator.name);
+	g_free (self->priv->accelerator);
+	g_free (self->priv->detailed_action);
 
 	G_OBJECT_CLASS (gth_script_parent_class)->finalize (base);
 }
 
 
+static char *
+detailed_action_from_id (char *id)
+{
+	GVariant *param;
+	char     *detailed_action;
+
+	param = g_variant_new_string (id);
+	detailed_action = g_action_print_detailed_name ("exec-script", param);
+
+	g_variant_unref (param);
+
+	return detailed_action;
+}
+
+
 static void
 gth_script_set_property (GObject      *object,
 		         guint         property_id,
@@ -172,6 +183,8 @@ gth_script_set_property (GObject      *object,
 		self->priv->id = g_value_dup_string (value);
 		if (self->priv->id == NULL)
 			self->priv->id = g_strdup ("");
+		g_free (self->priv->detailed_action);
+		self->priv->detailed_action = detailed_action_from_id (self->priv->id);
 		break;
 	case PROP_DISPLAY_NAME:
 		g_free (self->priv->display_name);
@@ -198,10 +211,8 @@ gth_script_set_property (GObject      *object,
 		self->priv->wait_command = g_value_get_boolean (value);
 		break;
 	case PROP_ACCELERATOR:
-		self->priv->accelerator.name = g_value_dup_string (value);
-		gtk_accelerator_parse (self->priv->accelerator.name,
-				       &self->priv->accelerator.keyval,
-				       &self->priv->accelerator.modifiers);
+		g_free (self->priv->accelerator);
+		self->priv->accelerator = g_value_dup_string (value);
 		break;
 	default:
 		break;
@@ -242,7 +253,7 @@ gth_script_get_property (GObject    *object,
 		g_value_set_boolean (value, self->priv->wait_command);
 		break;
 	case PROP_ACCELERATOR:
-		g_value_set_string (value, self->priv->accelerator.name);
+		g_value_set_string (value, self->priv->accelerator);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -344,9 +355,12 @@ gth_script_init (GthScript *self)
 	self->priv->id = NULL;
 	self->priv->display_name = NULL;
 	self->priv->command = NULL;
-	self->priv->accelerator.name = NULL;
-	self->priv->accelerator.keyval = 0;
-	self->priv->accelerator.modifiers = 0;
+	self->priv->visible = FALSE;
+	self->priv->shell_script = FALSE;
+	self->priv->for_each_file = FALSE;
+	self->priv->wait_command = FALSE;
+	self->priv->accelerator = NULL;
+	self->priv->detailed_action = NULL;
 }
 
 
@@ -356,7 +370,7 @@ gth_script_new (void)
 	GthScript *script;
 	char      *id;
 
-	id = _g_rand_string (ID_LENGTH);
+	id = _g_str_random (ID_LENGTH);
 	script = (GthScript *) g_object_new (GTH_TYPE_SCRIPT, "id", id, NULL);
 	g_free (id);
 
@@ -385,6 +399,13 @@ gth_script_get_command (GthScript *script)
 }
 
 
+const char *
+gth_script_get_detailed_action (GthScript *self)
+{
+	return self->priv->detailed_action;
+}
+
+
 gboolean
 gth_script_is_visible (GthScript *script)
 {
@@ -557,7 +578,7 @@ get_basename_wo_ext_func (GthFileData *file_data)
 	char *basename_wo_ext;
 
 	basename = g_file_get_basename (file_data->file);
-	basename_wo_ext = _g_uri_remove_extension (basename);
+	basename_wo_ext = _g_path_remove_extension (basename);
 
 	g_free (basename);
 
@@ -572,7 +593,7 @@ get_ext_func (GthFileData *file_data)
 	char *ext;
 
 	path = g_file_get_path (file_data->file);
-	ext = g_strdup (_g_uri_get_file_extension (path));
+	ext = g_strdup (_g_path_get_extension (path));
 
 	g_free (path);
 
@@ -661,7 +682,7 @@ create_attribute_list (GList    *file_list,
 		if (value != NULL) {
 			char *tmp_value;
 
-			tmp_value = _g_utf8_replace (value, "[\r\n]", " ");
+			tmp_value = _g_utf8_replace_pattern (value, "[\r\n]", " ");
 			g_free (value);
 			value = tmp_value;
 		}
@@ -687,6 +708,21 @@ create_attribute_list (GList    *file_list,
 }
 
 
+static char *
+get_timestamp (void)
+{
+	GDateTime *now;
+	char      *str;
+
+	now = g_date_time_new_now_local ();
+	str = g_date_time_format (now, "%Y-%m-%d %H.%M.%S");
+
+	g_date_time_unref (now);
+
+	return str;
+}
+
+
 static gboolean
 command_line_eval_cb (const GMatchInfo *info,
 		      GString          *res,
@@ -709,6 +745,8 @@ command_line_eval_cb (const GMatchInfo *info,
 		r = create_file_list (replace_data->file_list, get_ext_func, replace_data->quote_values);
 	else if (strcmp (match, "%P") == 0)
 		r = create_file_list (replace_data->file_list, get_parent_func, replace_data->quote_values);
+	else if (strcmp (match, "%T") == 0)
+		r = get_timestamp ();
 	else if (strncmp (match, "%attr", 5) == 0) {
 		r = create_attribute_list (replace_data->file_list, match, replace_data->quote_values);
 		if (r == NULL)
@@ -869,7 +907,7 @@ _split_command_for_quotation (char *string)
 	int          n;
 
 	remainder = string;
-	s = _g_utf8_strstr (remainder, delimiter);
+	s = _g_utf8_find_str (remainder, delimiter);
 	if (s != NULL) {
 		gsize delimiter_size = strlen (delimiter);
 
@@ -894,7 +932,7 @@ _split_command_for_quotation (char *string)
 				string_list = g_slist_prepend (string_list, new_string);
 
 				remainder = s + 1 /* strlen("}") */;
-				s = _g_utf8_strstr (remainder, delimiter);
+				s = _g_utf8_find_str (remainder, delimiter);
 			}
 		}
 	}
@@ -960,10 +998,10 @@ gth_script_get_command_line (GthScript  *script,
 
 					if (n_param == 1) {
 						g_free (asked_value->prompt);
-						asked_value->prompt = _g_utf8_strstrip (value);
+						asked_value->prompt = _g_utf8_strip (value);
 					}
 					else if (n_param == 2)
-						asked_value->default_value = _g_utf8_strstrip (value);
+						asked_value->default_value = _g_utf8_strip (value);
 					else
 						g_assert_not_reached ();
 
@@ -989,7 +1027,7 @@ gth_script_get_command_line (GthScript  *script,
 
 	/* replace the parameters in the command line */
 
-	re = g_regex_new ("%U|%F|%B|%N|%E|%P|%ask({[^}]+}({[^}]+})?)?|%attr{[^}]+}", 0, 0, NULL);
+	re = g_regex_new ("%U|%F|%B|%N|%E|%P|%T|%ask({[^}]+}({[^}]+})?)?|%attr{[^}]+}", 0, 0, NULL);
 
 	replace_data->quote_values = FALSE;
 	replace_data->last_asked_value = replace_data->asked_values;
@@ -1025,12 +1063,25 @@ gth_script_get_command_line (GthScript  *script,
 }
 
 
-void
-gth_script_get_accelerator (GthScript       *self,
-			    guint           *keyval,
-			    GdkModifierType *modifiers)
+const char *
+gth_script_get_accelerator (GthScript *self)
 {
-	g_return_if_fail (GTH_IS_SCRIPT (self));
-	if (keyval) *keyval = self->priv->accelerator.keyval;
-	if (modifiers) *modifiers = self->priv->accelerator.modifiers;
+	g_return_val_if_fail (GTH_IS_SCRIPT (self), NULL);
+	return self->priv->accelerator;
+}
+
+
+GthShortcut *
+gth_script_create_shortcut (GthScript *self)
+{
+	GthShortcut *shortcut;
+
+	shortcut = gth_shortcut_new ("exec-script", g_variant_new_string (gth_script_get_id (self)));
+	shortcut->description = g_strdup (self->priv->display_name);
+	shortcut->context = GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER;
+	shortcut->category = GTH_SHORTCUT_CATEGORY_LIST_TOOLS;
+	gth_shortcut_set_accelerator (shortcut, self->priv->accelerator);
+	shortcut->default_accelerator = g_strdup ("");
+
+	return shortcut;
 }
