@@ -32,6 +32,7 @@
 #endif /* HAVE_GSTREAMER */
 #include "gth-slideshow.h"
 #include "gth-transition.h"
+#include "actions.h"
 
 #define HIDE_CURSOR_DELAY 1
 #define HIDE_PAUSED_SIGN_DELAY 1
@@ -64,7 +65,7 @@ struct _GthSlideshowPrivate {
 	ClutterActor          *paused_actor;
 	guint32                last_button_event_time;
 #endif
-	GdkPixbuf             *current_pixbuf;
+	GthImage              *current_image;
 	GtkWidget             *viewer;
 	guint                  next_event;
 	guint                  delay;
@@ -90,33 +91,16 @@ struct _GthSlideshowPrivate {
 
 G_DEFINE_TYPE_WITH_CODE (GthSlideshow,
 			 gth_slideshow,
-			 GTK_TYPE_WINDOW,
+			 GTH_TYPE_WINDOW,
 			 G_ADD_PRIVATE (GthSlideshow))
 
 
-static void
-_gth_slideshow_close_cb (gpointer user_data)
-{
-	GthSlideshow *self = user_data;
-	gboolean      close_browser;
-	GthBrowser   *browser;
-
-	browser = self->priv->browser;
-	close_browser = ! gtk_widget_get_visible (GTK_WIDGET (browser));
-	self->priv->projector->show_cursor (self);
-	self->priv->projector->finalize (self);
-	gtk_widget_destroy (GTK_WIDGET (self));
-
-	if (close_browser)
-		gth_window_close (GTH_WINDOW (browser));
-}
-
-
-static void
-_gth_slideshow_close (GthSlideshow *self)
-{
-	call_when_idle (_gth_slideshow_close_cb, self);
-}
+static const GActionEntry actions[] = {
+	{ "slideshow-close", gth_slideshow_activate_close },
+	{ "slideshow-toggle-pause", gth_slideshow_activate_toggle_pause },
+	{ "slideshow-next-image", gth_slideshow_activate_next_image },
+	{ "slideshow-previous-image", gth_slideshow_activate_previous_image },
+};
 
 
 static int
@@ -141,9 +125,6 @@ _gth_slideshow_reset_current (GthSlideshow *self)
 }
 
 
-static void _gth_slideshow_load_next_image (GthSlideshow *self);
-
-
 static void
 preloader_load_ready_cb (GObject	*source_object,
 			 GAsyncResult	*result,
@@ -167,20 +148,20 @@ preloader_load_ready_cb (GObject	*source_object,
 					       &error))
 	{
 		g_clear_error (&error);
-		_gth_slideshow_load_next_image (self);
+		gth_slideshow_load_next_image (self);
 		return;
 	}
 
-	_g_object_unref (self->priv->current_pixbuf);
-	self->priv->current_pixbuf = gth_image_get_pixbuf (image);
+	_g_object_unref (self->priv->current_image);
+	self->priv->current_image = _g_object_ref (image);
 
-	if (self->priv->current_pixbuf == NULL) {
-		_gth_slideshow_load_next_image (self);
+	if (self->priv->current_image == NULL) {
+		gth_slideshow_load_next_image (self);
 		return;
 	}
 
 	self->priv->one_loaded = TRUE;
-	self->priv->projector->image_ready (self, self->priv->current_pixbuf);
+	self->priv->projector->image_ready (self, self->priv->current_image);
 
 	_g_object_unref (requested);
 	_g_object_unref (image);
@@ -203,7 +184,7 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 
 	if (self->priv->current == NULL) {
 		if (! self->priv->one_loaded || ! self->priv->wrap_around) {
-			_gth_slideshow_close (self);
+			gth_slideshow_close (self);
 			return;
 		}
 		_gth_slideshow_reset_current (self);
@@ -232,34 +213,6 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 }
 
 
-static void
-_gth_slideshow_load_next_image (GthSlideshow *self)
-{
-	self->priv->projector->load_next_image (self);
-	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
-
-	if (self->priv->paused)
-		return;
-
-	self->priv->current = self->priv->current->next;
-	_gth_slideshow_load_current_image (self);
-}
-
-
-static void
-_gth_slideshow_load_prev_image (GthSlideshow *self)
-{
-	self->priv->projector->load_prev_image (self);
-	self->priv->direction = GTH_SLIDESHOW_DIRECTION_BACKWARD;
-
-	if (self->priv->paused)
-		return;
-
-	self->priv->current = self->priv->current->prev;
-	_gth_slideshow_load_current_image (self);
-}
-
-
 static gboolean
 next_image_cb (gpointer user_data)
 {
@@ -269,7 +222,7 @@ next_image_cb (gpointer user_data)
 		g_source_remove (self->priv->next_event);
 		self->priv->next_event = 0;
 	}
-	_gth_slideshow_load_next_image (self);
+	gth_slideshow_load_next_image (self);
 
 	return FALSE;
 }
@@ -281,7 +234,7 @@ view_next_image_automatically (GthSlideshow *self)
 	if (self->priv->automatic && ! self->priv->paused)
 		gth_screensaver_inhibit (self->priv->screensaver,
 					 GTK_WINDOW (self),
-					 _("Playing slideshow"));
+					 _("Playing a presentation"));
 	else
 		gth_screensaver_uninhibit (self->priv->screensaver);
 
@@ -304,7 +257,7 @@ gth_slideshow_finalize (GObject *object)
 		g_source_remove (self->priv->hide_cursor_event);
 
 	_g_object_unref (self->priv->pause_pixbuf);
-	_g_object_unref (self->priv->current_pixbuf);
+	_g_object_unref (self->priv->current_image);
 	_g_object_list_unref (self->priv->file_list);
 	_g_object_unref (self->priv->browser);
 	_g_object_unref (self->priv->preloader);
@@ -352,27 +305,6 @@ hide_cursor_cb (gpointer data)
 }
 
 
-static void
-_gth_slideshow_toggle_pause (GthSlideshow *self)
-{
-	self->priv->paused = ! self->priv->paused;
-	if (self->priv->paused) {
-		self->priv->projector->paused (self);
-#if HAVE_GSTREAMER
-		if (self->priv->playbin != NULL)
-			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
-#endif
-	}
-	else { /* resume */
-		_gth_slideshow_load_next_image (self);
-#if HAVE_GSTREAMER
-		if (self->priv->playbin != NULL)
-			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
-#endif
-	}
-}
-
-
 #if HAVE_GSTREAMER
 
 
@@ -446,6 +378,18 @@ gth_slideshow_show_cb (GtkWidget    *widget,
 }
 
 
+static gboolean
+_gth_slideshow_key_press_cb (GthSlideshow *self,
+			     GdkEventKey  *event,
+			     gpointer      user_data)
+{
+	return gth_window_activate_shortcut (GTH_WINDOW (self),
+					     GTH_SHORTCUT_CONTEXT_SLIDESHOW,
+					     event->keyval,
+					     event->state);
+}
+
+
 static void
 gth_slideshow_init (GthSlideshow *self)
 {
@@ -464,7 +408,7 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->animating = FALSE;
 	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
 	self->priv->random_order = FALSE;
-	self->priv->current_pixbuf = NULL;
+	self->priv->current_image = NULL;
 	self->priv->screensaver = gth_screensaver_new (NULL);
 	self->priv->preloader = gth_image_preloader_new ();
 }
@@ -494,7 +438,20 @@ _gth_slideshow_construct (GthSlideshow *self,
 
 	self->priv->projector->construct (self);
 
+	g_action_map_add_action_entries (G_ACTION_MAP (self),
+					 actions,
+					 G_N_ELEMENTS (actions),
+					 self);
+	gth_window_copy_shortcuts (GTH_WINDOW (self),
+				   GTH_WINDOW (self->priv->browser),
+				   GTH_SHORTCUT_CONTEXT_SLIDESHOW);
+
 	g_signal_connect (self, "show", G_CALLBACK (gth_slideshow_show_cb), self);
+
+	g_signal_connect (self,
+			  "key-press-event",
+			  G_CALLBACK (_gth_slideshow_key_press_cb),
+			  NULL);
 }
 
 
@@ -565,6 +522,98 @@ gth_slideshow_set_random_order (GthSlideshow *self,
 }
 
 
+void
+gth_slideshow_toggle_pause (GthSlideshow *self)
+{
+	g_return_if_fail (GTH_IS_SLIDESHOW (self));
+
+	self->priv->paused = ! self->priv->paused;
+	if (self->priv->paused) {
+		self->priv->projector->paused (self);
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
+#endif
+	}
+	else { /* resume */
+		gth_slideshow_load_next_image (self);
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+#endif
+	}
+}
+
+
+void
+gth_slideshow_load_next_image (GthSlideshow *self)
+{
+	g_return_if_fail (GTH_IS_SLIDESHOW (self));
+
+	self->priv->projector->load_next_image (self);
+	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
+
+	if (self->priv->paused)
+		return;
+
+	self->priv->current = self->priv->current->next;
+	_gth_slideshow_load_current_image (self);
+}
+
+
+void
+gth_slideshow_load_prev_image (GthSlideshow *self)
+{
+	g_return_if_fail (GTH_IS_SLIDESHOW (self));
+
+	self->priv->projector->load_prev_image (self);
+	self->priv->direction = GTH_SLIDESHOW_DIRECTION_BACKWARD;
+
+	if (self->priv->paused)
+		return;
+
+	self->priv->current = self->priv->current->prev;
+	_gth_slideshow_load_current_image (self);
+}
+
+
+void
+gth_slideshow_next_image_or_resume (GthSlideshow *self)
+{
+	g_return_if_fail (GTH_IS_SLIDESHOW (self));
+
+	if (self->priv->paused)
+		gth_slideshow_toggle_pause (self);
+	else
+		gth_slideshow_load_next_image (self);
+}
+
+
+static void
+_gth_slideshow_close_cb (gpointer user_data)
+{
+	GthSlideshow *self = user_data;
+	gboolean      close_browser;
+	GthBrowser   *browser;
+
+	browser = self->priv->browser;
+	close_browser = ! gtk_widget_get_visible (GTK_WIDGET (browser));
+	self->priv->projector->show_cursor (self);
+	self->priv->projector->finalize (self);
+	gtk_widget_destroy (GTK_WIDGET (self));
+
+	if (close_browser)
+		gth_window_close (GTH_WINDOW (browser));
+}
+
+
+void
+gth_slideshow_close (GthSlideshow *self)
+{
+	call_when_idle (_gth_slideshow_close_cb, self);
+}
+
+
 /* -- default projector -- */
 
 
@@ -584,9 +633,9 @@ default_projector_load_prev_image (GthSlideshow *self)
 
 static void
 default_projector_image_ready (GthSlideshow *self,
-			       GdkPixbuf    *pixbuf)
+			       GthImage     *image)
 {
-	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf, -1, -1);
+	gth_image_viewer_set_image (GTH_IMAGE_VIEWER (self->priv->viewer), image, -1, -1);
 	view_next_image_automatically (self);
 }
 
@@ -641,50 +690,15 @@ viewer_event_cb (GtkWidget    *widget,
 	else if (event->type == GDK_BUTTON_PRESS) {
 		switch (((GdkEventButton *) event)->button) {
 		case 1:
-			_gth_slideshow_load_next_image (self);
+			gth_slideshow_load_next_image (self);
 			break;
 		case 3:
-			_gth_slideshow_load_prev_image (self);
+			gth_slideshow_load_prev_image (self);
 			break;
 		default:
 			break;
 		}
 	}
-	else if (event->type == GDK_KEY_PRESS) {
-		switch (((GdkEventKey *) event)->keyval) {
-		case GDK_KEY_F5:
-			_gth_slideshow_close (self);
-			break;
-		}
-	}
-	else if (event->type == GDK_KEY_RELEASE) {
-		switch (((GdkEventKey *) event)->keyval) {
-		case GDK_KEY_Escape:
-		case GDK_KEY_q:
-			_gth_slideshow_close (self);
-			break;
-		case GDK_KEY_p:
-			_gth_slideshow_toggle_pause (self);
-			break;
-
-		case GDK_KEY_space:
-		case GDK_KEY_Down:
-		case GDK_KEY_Right:
-		case GDK_KEY_Page_Down:
-			if (self->priv->paused)
-				_gth_slideshow_toggle_pause (self);
-			else
-				_gth_slideshow_load_next_image (self);
-			break;
-
-		case GDK_KEY_BackSpace:
-		case GDK_KEY_Up:
-		case GDK_KEY_Left:
-		case GDK_KEY_Page_Up:
-			_gth_slideshow_load_prev_image (self);
-			break;
-		}
-	}
 }
 
 
@@ -744,6 +758,7 @@ default_projector_construct (GthSlideshow *self)
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_CHANGE_FIT_SIZE);
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_QUALITY_LOW);
 	gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), default_projector_pause_painter, self);
+	gth_image_viewer_set_transparency_style (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_TRANSPARENCY_STYLE_CHECKERED);
 
 	g_signal_connect (self->priv->viewer, "button-press-event", G_CALLBACK (viewer_event_cb), self);
 	g_signal_connect (self->priv->viewer, "motion-notify-event", G_CALLBACK (viewer_event_cb), self);
@@ -916,8 +931,9 @@ _gth_slideshow_get_transition (GthSlideshow *self)
 
 static void
 clutter_projector_image_ready (GthSlideshow *self,
-			       GdkPixbuf    *pixbuf)
+			       GthImage     *image_data)
 {
+	GdkPixbuf    *pixbuf;
 	GdkPixbuf    *image;
 	ClutterActor *texture;
 	int           pixbuf_w, pixbuf_h;
@@ -928,6 +944,7 @@ clutter_projector_image_ready (GthSlideshow *self,
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
+	pixbuf = gth_image_get_pixbuf (image_data);
 	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
 				FALSE,
 				gdk_pixbuf_get_bits_per_sample (pixbuf),
@@ -976,6 +993,7 @@ clutter_projector_image_ready (GthSlideshow *self,
 		clutter_timeline_advance (self->priv->timeline, GTH_TRANSITION_DURATION);
 
 	g_object_unref (image);
+	g_object_unref (pixbuf);
 }
 
 
@@ -1051,50 +1069,15 @@ stage_input_cb (ClutterStage *stage,
 
 		switch (clutter_event_get_button (event)) {
 		case 1:
-			_gth_slideshow_load_next_image (self);
+			gth_slideshow_load_next_image (self);
 			break;
 		case 3:
-			_gth_slideshow_load_prev_image (self);
+			gth_slideshow_load_prev_image (self);
 			break;
 		default:
 			break;
 		}
 	}
-	else if (event->type == CLUTTER_KEY_PRESS) {
-		switch (clutter_event_get_key_symbol (event)) {
-		case CLUTTER_KEY_F5:
-			_gth_slideshow_close (self);
-			break;
-		}
-	}
-	else if (event->type == CLUTTER_KEY_RELEASE) {
-		switch (clutter_event_get_key_symbol (event)) {
-		case CLUTTER_KEY_Escape:
-		case CLUTTER_KEY_q:
-			_gth_slideshow_close (self);
-			break;
-		case CLUTTER_KEY_p:
-			_gth_slideshow_toggle_pause (self);
-			break;
-
-		case CLUTTER_KEY_space:
-		case CLUTTER_KEY_Down:
-		case CLUTTER_KEY_Right:
-		case CLUTTER_KEY_Page_Down:
-			if (self->priv->paused)
-				_gth_slideshow_toggle_pause (self);
-			else
-				_gth_slideshow_load_next_image (self);
-			break;
-
-		case CLUTTER_KEY_BackSpace:
-		case CLUTTER_KEY_Up:
-		case CLUTTER_KEY_Left:
-		case CLUTTER_KEY_Page_Up:
-			_gth_slideshow_load_prev_image (self);
-			break;
-		}
-	}
 }
 
 
@@ -1102,6 +1085,7 @@ static void
 adapt_image_size_to_stage_size (GthSlideshow *self)
 {
 	gfloat          stage_w, stage_h;
+	GdkPixbuf      *pixbuf;
 	GdkPixbuf      *image;
 	int             pixbuf_w, pixbuf_h;
 	int             pixbuf_x, pixbuf_y;
@@ -1114,23 +1098,24 @@ adapt_image_size_to_stage_size (GthSlideshow *self)
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
-	if (self->priv->current_pixbuf == NULL)
+	if (self->priv->current_image == NULL)
 		return;
 
-	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (self->priv->current_pixbuf),
+	pixbuf = gth_image_get_pixbuf (self->priv->current_image);
+	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
 				FALSE,
-				gdk_pixbuf_get_bits_per_sample (self->priv->current_pixbuf),
+				gdk_pixbuf_get_bits_per_sample (pixbuf),
 				stage_w,
 				stage_h);
 	gdk_pixbuf_fill (image, 0x000000ff);
 
-	pixbuf_w = gdk_pixbuf_get_width (self->priv->current_pixbuf);
-	pixbuf_h = gdk_pixbuf_get_height (self->priv->current_pixbuf);
+	pixbuf_w = gdk_pixbuf_get_width (pixbuf);
+	pixbuf_h = gdk_pixbuf_get_height (pixbuf);
 	scale_keeping_ratio (&pixbuf_w, &pixbuf_h, (int) stage_w, (int) stage_h, TRUE);
 	pixbuf_x = (stage_w - pixbuf_w) / 2;
 	pixbuf_y = (stage_h - pixbuf_h) / 2;
 
-	gdk_pixbuf_composite (self->priv->current_pixbuf,
+	gdk_pixbuf_composite (pixbuf,
 			      image,
 			      pixbuf_x,
 			      pixbuf_y,
@@ -1138,8 +1123,8 @@ adapt_image_size_to_stage_size (GthSlideshow *self)
 			      pixbuf_h,
 			      pixbuf_x,
 			      pixbuf_y,
-			      (double) pixbuf_w / gdk_pixbuf_get_width (self->priv->current_pixbuf),
-			      (double) pixbuf_h / gdk_pixbuf_get_height (self->priv->current_pixbuf),
+			      (double) pixbuf_w / gdk_pixbuf_get_width (pixbuf),
+			      (double) pixbuf_h / gdk_pixbuf_get_height (pixbuf),
 			      GDK_INTERP_BILINEAR,
 			      255);
 
@@ -1156,6 +1141,7 @@ adapt_image_size_to_stage_size (GthSlideshow *self)
 	_gth_slideshow_reset_textures_position (self);
 
 	g_object_unref (image);
+	g_object_unref (pixbuf);
 }
 
 
