@@ -428,7 +428,8 @@ _cairo_image_surface_scale_for_thumbnail (cairo_surface_t *image,
 	cairo_surface_t *scaled;
 
 	scaled = _cairo_image_surface_scale (image, new_width, new_height, SCALE_FILTER_GOOD, NULL);
-	_cairo_image_surface_copy_metadata (image, scaled);
+	if (scaled != NULL)
+		_cairo_image_surface_copy_metadata (image, scaled);
 
 	return scaled;
 }
@@ -513,7 +514,7 @@ is_a_cache_file (const char *uri)
 
 	cache_dir_1 = g_build_filename (g_get_home_dir (), ".thumbnails", NULL);
 	cache_dir_2 = g_build_filename (g_get_user_cache_dir (), "thumbnails", NULL);
-	result = _g_uri_parent_of_uri (cache_dir_1, filename) || _g_uri_parent_of_uri (cache_dir_2, filename);
+	result = _g_path_is_parent (cache_dir_1, filename) || _g_path_is_parent (cache_dir_2, filename);
 
 	g_free (cache_dir_1);
 	g_free (cache_dir_2);
@@ -738,6 +739,41 @@ watch_thumbnailer_cb (GPid     pid,
 }
 
 
+static void
+load_with_system_thumbnailer (GthThumbLoader *self,
+			      LoadData       *load_data)
+{
+	char   *uri;
+	GError *error = NULL;
+
+	uri = g_file_get_uri (load_data->file_data->file);
+	if (gnome_desktop_thumbnail_factory_generate_from_script (self->priv->thumb_factory,
+								  uri,
+								  gth_file_data_get_mime_type (load_data->file_data),
+								  &load_data->thumbnailer_pid,
+								  &load_data->thumbnailer_tmpfile,
+								  &error))
+	{
+		load_data->thumbnailer_watch = g_child_watch_add (load_data->thumbnailer_pid,
+								  watch_thumbnailer_cb,
+								  load_data);
+		load_data->thumbnailer_timeout = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
+								kill_thumbnailer_cb,
+								load_data);
+		load_data->cancellable_watch = g_timeout_add (CHECK_CANCELLABLE_DELAY,
+							      check_cancellable_cb,
+							      load_data);
+	}
+	else {
+		g_clear_error (&error);
+		failed_to_load_original_image (self, load_data);
+		load_data_unref (load_data);
+	}
+
+	g_free (uri);
+}
+
+
 static void
 original_image_ready_cb (GObject      *source_object,
 		         GAsyncResult *res,
@@ -762,45 +798,23 @@ original_image_ready_cb (GObject      *source_object,
 		/* error loading the original image, try with the system
 		 * thumbnailer */
 
-		char *uri;
-
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 			g_task_return_error (load_data->task, error);
 			return;
 		}
 
 		g_clear_error (&error);
-
-		uri = g_file_get_uri (load_data->file_data->file);
-		if (gnome_desktop_thumbnail_factory_generate_from_script (self->priv->thumb_factory,
-									  uri,
-									  gth_file_data_get_mime_type (load_data->file_data),
-									  &load_data->thumbnailer_pid,
-									  &load_data->thumbnailer_tmpfile,
-									  &error))
-		{
-			load_data->thumbnailer_watch = g_child_watch_add (load_data->thumbnailer_pid,
-									  watch_thumbnailer_cb,
-									  load_data);
-			load_data->thumbnailer_timeout = g_timeout_add (MAX_THUMBNAILER_LIFETIME,
-									kill_thumbnailer_cb,
-									load_data);
-			load_data->cancellable_watch = g_timeout_add (CHECK_CANCELLABLE_DELAY,
-								      check_cancellable_cb,
-								      load_data);
-		}
-		else {
-			g_clear_error (&error);
-			failed_to_load_original_image (self, load_data);
-			load_data_unref (load_data);
-		}
-
-		g_free (uri);
-
+		load_with_system_thumbnailer (self, load_data);
 		return;
 	}
 
 	surface = gth_image_get_cairo_surface (image);
+	if (surface == NULL) {
+		g_object_unref (image);
+		load_with_system_thumbnailer (self, load_data);
+		return;
+	}
+
 	original_image_loaded_correctly (self,
 					 load_data,
 					 surface,
